@@ -57,7 +57,7 @@ function waitAndDraw(milliseconds) {
             cb();
     });
 
-    return callback => cb = callback;
+    return callback => (cb = callback);
 }
 
 function waitSignal(object, signal) {
@@ -69,7 +69,7 @@ function waitSignal(object, signal) {
             cb();
     });
 
-    return callback => cb = callback;
+    return callback => (cb = callback);
 }
 
 function extractBootTimestamp() {
@@ -137,9 +137,9 @@ function *run() {
     Main.overview.hide();
     yield Scripting.waitLeisure();
 
-    ////////////////////////////////////////
-    // Tests of redraw speed
-    ////////////////////////////////////////
+    // --------------------- //
+    // Tests of redraw speed //
+    // --------------------- //
 
     global.frame_timestamps = true;
     global.frame_finish_timestamp = true;
@@ -186,8 +186,6 @@ function *run() {
 
     yield Scripting.sleep(1000);
 
-    ////////////////////////////////////////
-
     let appSys = Shell.AppSystem.get_default();
     let app = appSys.lookup_app('org.gnome.gedit.desktop');
 
