@@ -31,7 +31,7 @@ function addBackgroundMenu(actor, layoutManager) {
 
     function openMenu(x, y) {
         Main.layoutManager.setDummyCursorGeometry(x, y, 0, 0);
-        actor._backgroundMenu.open(BoxPointer.PopupAnimation.NONE);
+        actor._backgroundMenu.open(BoxPointer.PopupAnimation.FULL);
     }
 
     let clickAction = new Clutter.ClickAction();
