@@ -63,7 +63,7 @@ function startAppForMount(app, mount) {
     files.push(root);
 
     try {
-        retval = app.launch(files, 
+        retval = app.launch(files,
                             global.create_app_launch_context(0, -1));
     } catch (e) {
         log(`Unable to launch the application ${app.get_name()}: ${e}`);
@@ -72,8 +72,6 @@ function startAppForMount(app, mount) {
     return retval;
 }
 
-/******************************************/
-
 const HotplugSnifferIface = loadInterfaceXML('org.gnome.Shell.HotplugSniffer');
 const HotplugSnifferProxy = Gio.DBusProxy.makeProxyWrapper(HotplugSnifferIface);
 function HotplugSniffer() {
@@ -117,9 +115,9 @@ var ContentTypeDiscoverer = class {
 
             let hotplugSniffer = new HotplugSniffer();
             hotplugSniffer.SniffURIRemote(root.get_uri(),
-                 ([contentTypes]) => {
-                     this._emitCallback(mount, contentTypes);
-                 });
+                ([contentTypes]) => {
+                    this._emitCallback(mount, contentTypes);
+                });
         }
     }
 
@@ -215,11 +213,11 @@ var AutorunDispatcher = class {
     }
 
     _addSource(mount, apps) {
-        // if we already have a source showing for this 
+        // if we already have a source showing for this
         // mount, return
         if (this._getSourceForMount(mount))
             return;
-     
+
         // add a new source
         this._sources.push(new AutorunSource(this._manager, mount, apps));
     }
@@ -264,7 +262,7 @@ var AutorunDispatcher = class {
 
     removeMount(mount) {
         let source = this._getSourceForMount(mount);
-        
+
         // if we aren't tracking this mount, don't do anything
         if (!source)
             return;
@@ -325,10 +323,10 @@ var AutorunNotification = class extends MessageTray.Notification {
                                  style_class: 'hotplug-notification-item-icon' });
         box.add(icon);
 
-        let label = new St.Bin({ y_align: St.Align.MIDDLE,
-                                 child: new St.Label
-                                 ({ text: _("Open with %s").format(app.get_name()) })
-                               });
+        let label = new St.Bin({
+            y_align: St.Align.MIDDLE,
+            child: new St.Label({ text: _("Open with %s").format(app.get_name()) }),
+        });
         box.add(label);
 
         let button = new St.Button({ child: box,
