@@ -94,15 +94,17 @@ class RunDialog extends ModalDialog.ModalDialog {
 
         this._errorBox.hide();
 
-        this.setButtons([{ action: this.close.bind(this),
-                           label: _("Close"),
-                           key: Clutter.Escape }]);
+        this.setButtons([{
+            action: this.close.bind(this),
+            label: _("Close"),
+            key: Clutter.Escape,
+        }]);
 
         this._pathCompleter = new Gio.FilenameCompleter();
 
         this._history = new History.HistoryManager({ gsettingsKey: HISTORY_KEY,
                                                      entry: this._entryText });
-        this._entryText.connect('activate', (o) => {
+        this._entryText.connect('activate', o => {
             this.popModal();
             this._run(o.get_text(),
                       Clutter.get_current_event().get_state() & Clutter.ModifierType.CONTROL_MASK);
